
;Schulstundenzhler (LCD 2x16, DCF)

; ,------------------,
; | H1 hh:mm pp.p% . |
; | 2004-09-30 22:09 |
; '------------------'

;PB0-7 = LCD Datenbus

;PD5   = LCD RS
;PD6   = LCD E

;v1   2004-09-14..2004-09-30   Arne Rossius

;===============================================================================

.include "..\2313def.inc"

;R0 needed for lpm instruction

.def	op1a	= R1
.def	op1b	= R2
.def	op1c	= R3
.def	op1d	= R4
.def	op2a	= R5
.def	op2b	= R6
.def	op3a	= R7
.def	op3b	= R8
.def	op3c	= R9
.def	op3d	= R10
.def	op4a	= R11
.def	op4b	= R12
.def	op4c	= R13
.def	op4d	= R14

.def	temp	= R16
.def	temp2	= R17
.def	w1	= R18
.def	w2	= R19
.def	begin	= R20
.def	count	= R21

.def	dcfbit	= R22
.def	dcfcnt	= R23
.def	flags	= R24
.def	subsec	= R25
.def	sek	= R26

.def	op5a	= R27
.def	op5b	= R28

;R30/R31 needed as Z register

.equ	DBase	= 0x60		;RAM Base Adress for DCF Bits (0x60 for SRAM, 0x00 for R0-R13)
.equ	Base	= DBase+8	;RAM Base Adress for verified Time/Date information
.equ	nBase	= Base+8	;RAM Base Adress for temporary Time/Date information


.equ	RS	= 5
.equ	E	= 6

.equ	H1	=  8*3600 +  0*60
.equ	P1	=  8*3600 + 45*60
.equ	H2	=  8*3600 + 50*60
.equ	P2	=  9*3600 + 35*60
.equ	H3	=  9*3600 + 50*60
.equ	P3	= 10*3600 + 35*60
.equ	H4	= 10*3600 + 40*60
.equ	P4	= 11*3600 + 25*60
.equ	H5	= 11*3600 + 40*60
.equ	P5	= 12*3600 + 25*60
.equ	H6	= 12*3600 + 30*60
.equ	Ende	= 13*3600 + 15*60

;-------------------------------------------------------------------------------

.cseg

.org $0400-(10*7)/2
.db "  MONTAG  "
.db " DIENSTAG "
.db " MITTWOCH "
.db "DONNERSTAG"
.db " FREITAG  "
.db " SAMSTAG  "
.db " SONNTAG  "

.org $0000
	rjmp	start
	
	nop
	nop
	nop
	nop
	nop
	
.org $0006				;timer0 overflow (16Hz)
	rjmp	int16
;.org $0008				;UDR empty (send next char)
;	rjmp	intudr
	
;-------------------------------------------------------------------------------

start:
	ldi	temp,	low(RAMEND)
	out	SPL,	temp

	ldi	temp,	0xFF
	out	DDRB,	temp
	ldi	temp,	0x00
	out	PORTB,	temp
	ldi	temp,	0x60
	out	DDRD,	temp
	ldi	temp,	0x1F
	out	PORTD,	temp

	ldi	temp,	(1<<TOIE0)	;timer0: overflow int, Clk/1024
	out	TIMSK,	temp
	ldi	temp,	0x05
	out	TCCR0,	temp
	
	ldi	temp,	0x80		;turn off analog comperator
	out	ACSR,	temp

	clr	dcfbit			;Init
	clr	dcfcnt
	clr	flags
	ldi	subsec,	0xFF
	clr	sek

	ldi	w1,	200
	rcall	wait

	rcall	lcdinit

	sei

;--------------------

loop:
	in	temp,	MCUCR
	ori	temp,	(1<<SE)
	out	MCUCR,	temp
	sleep
	rjmp	loop

;===============================================================================

lcdinit:				;Initialisiert und lscht das LCD
	ldi	temp,	0x30
	rcall	lcdcmd
	ldi	w1,	50
	rcall	wait
	rcall	lcdcmd
	rcall	lcdcmd
	ldi	temp,	0x38
	rcall	lcdcmd
	ldi	temp,	0x08
	rcall	lcdcmd
	ldi	temp,	0x01
	rcall	lcdcmd
	ldi	temp,	0x06
	rcall	lcdcmd
	
	ldi	w1,	50
	rcall	wait
	
	ldi	temp,	0x0C
	rcall	lcdcmd
	ret

;--------------------

lcdcmd:					;Sendet den Befehl in temp ans LCD
	out	PORTB,	temp
	cbi	PORTD,	RS
	nop
	nop
	sbi	PORTD,	E
	nop
	nop
	cbi	PORTD,	E
	ldi	w1,	1
	rcall	wait
	ret

;--------------------
	
lcdchr:					;Sendet das Zeichen in temp ans LCD
	out	PORTB,	temp
	sbi	PORTD,	RS
	nop
	nop
	sbi	PORTD,	E
	nop
	nop
	cbi	PORTD,	E
	ldi	w1,	1
	rcall	wait
	ret

;--------------------

wait:					;Wartet w1 * ~100s @ 4.194304MHz
	ldi	w2,	0x85
wloop:
	dec	w2
	brne	wloop
	ldi	w2,	0x85
	dec	w1
	brne	wloop
	ret

;===============================================================================

int16:
	push	temp
	in	temp,	SREG
	push	temp
	
	cpi	subsec,	0xFF		;increment time
	breq	inctimeend
	inc	subsec
	cpi	subsec,	0x10
	brlo	inctimeend
	clr	subsec
	inc	sek
	
	cpi	sek,	60	;minute
	brlo	newsecond
	clr	sek
	ldi	ZL,	Base+0
	ld	temp2,	Z
	inc	temp2
	mov	temp,	temp2
	st	Z,	temp2
	andi	temp2,	0x0F
	cpi	temp2,	0x0A
	brlo	newminute
	andi	temp,	0xF0
	swap	temp
	inc	temp
	swap	temp
	st	Z,	temp
	
	cpi	temp,	0x60	;hour
	brlo	newminute
	clr	temp
	st	Z,	temp
	ldi	ZL,	Base+1
	ld	temp2,	Z
	inc	temp2
	mov	temp,	temp2
	st	Z,	temp2
	andi	temp2,	0x0F
	cpi	temp2,	0x0A
	brlo	chkhrs24
	andi	temp,	0xF0
	swap	temp
	inc	temp
	swap	temp
	st	Z,	temp
	rjmp	newminute
chkhrs24:
	cpi	temp,	0x24	;weekdays/days
	brlo	newminute
	clr	temp
	st	Z,	temp
	andi	flags,	0xFD

	;Increment Date/Weekday here in future releases
	
newminute:
	rcall	displaytime
	
newsecond:
	rcall	displaypercent
	
inctimeend:



	in	temp,	PIND		;receive DCF signal
	ldi	temp2,	0xFF
	eor	temp,	temp2
	andi	temp,	0x01
	
	mov	temp2,	flags	;input changed?
	andi	temp2,	0x01
	cp	temp,	temp2
	breq	nochange
	andi	flags,	0xFE
	sbrc	temp,	0
	ori	flags,	0x01

	cpi	temp,	1	;0 -> 1
	brne	negedge
	;sbi	PORTD,	2
	
	ldi	temp,	0x8F
	rcall	lcdcmd
	ldi	temp,	'.'
	rcall	lcdchr

	cpi	dcfcnt,	20
	brsh	endofminute
	rjmp	edgeend
	
negedge:			;1 -> 0
	;cbi	PORTD,	2

	ldi	temp,	0x8F
	rcall	lcdcmd
	ldi	temp,	' '
	rcall	lcdchr
	
	mov	temp,	dcfbit		;save received DCF bit
	andi	temp,	0x07
	ldi	temp2,	0x01
ramdcfbit:
	tst	temp
	breq	ramdcfbitend
	lsl	temp2
	dec	temp
	rjmp	ramdcfbit
ramdcfbitend:
	mov	temp,	dcfbit
	lsr	temp
	lsr	temp
	lsr	temp
		
	subi	temp,	-DBase
	ldi	ZH,	0
	mov	ZL,	temp
	ld	temp,	Z
	or	temp,	temp2

	cpi	dcfcnt,	2
	brsh	saveend
	com	temp2
	and	temp,	temp2
saveend:
	st	Z,	temp
	inc	dcfbit
	cpi	dcfbit,	60
	brlo	edgeend
	;cbi	PORTD,	3
	clr	dcfbit
	rjmp	edgeend	
	
endofminute:				;end of minute, evaluate received DCF data if correct number of bits
	cpi	dcfbit,	59
	brne	error
	rcall	evaluate
	clr	dcfbit
	rjmp	edgeend
error:
	clr	dcfbit
	;cbi	PORTD,	3
	
edgeend:
	clr	dcfcnt
	rjmp	dcfend

nochange:
	inc	dcfcnt
	cpi	dcfcnt,	40
	brlo	dcfend
	;cbi	PORTD,	3
	clr	dcfcnt
	clr	dcfbit

dcfend:



int16end:	
	pop	temp
	out	SREG,	temp
	pop	temp
	reti
	
;--------------------

evaluate:				;dcfbit used as third temp register
	ldi	ZH,	0	;Minutes
	ldi	ZL,	DBase+2
	ld	temp2,	Z
	lsr	temp2
	lsr	temp2
	lsr	temp2
	lsr	temp2
	lsr	temp2
	ldi	ZL,	DBase+3
	ld	temp,	Z
	andi	temp,	0x0F
	lsl	temp
	lsl	temp
	lsl	temp
	add	temp2,	temp
	ldi	ZL,	nBase+0
	st	Z,	temp2
	clr	dcfbit
	rcall	checksum
	
	ldi	ZL,	DBase+3	;Minutes Parity
	ld	temp,	Z
	andi	temp,	0x10
	swap	temp
	andi	dcfbit,	0x01
	cpse	temp,	dcfbit
	rjmp	parityerror
	
	ldi	ZL,	DBase+3	;Hours
	ld	temp2,	Z
	lsr	temp2
	lsr	temp2
	lsr	temp2
	lsr	temp2
	lsr	temp2
	ldi	ZL,	DBase+4
	ld	temp,	Z
	andi	temp,	0x07
	lsl	temp
	lsl	temp
	lsl	temp
	add	temp2,	temp
	ldi	ZL,	nBase+1
	st	Z,	temp2
	clr	dcfbit
	rcall	checksum
	
	ldi	ZL,	DBase+4	;Hours Parity
	ld	temp,	Z
	andi	temp,	0x08
	lsr	temp
	lsr	temp
	lsr	temp
	andi	dcfbit,	0x01
	cpse	temp,	dcfbit
	rjmp	parityerror

	ldi	ZL,	DBase+4	;Days
	ld	temp2,	Z
	andi	temp2,	0xF0
	ldi	ZL,	DBase+5
	ld	temp,	Z
	andi	temp,	0x03
	add	temp2,	temp
	swap	temp2
	ldi	ZL,	nBase+2
	st	Z,	temp2
	clr	dcfbit
	rcall	checksum

	ldi	ZL,	DBase+5	;Weekday
	ld	temp2,	Z
	andi	temp2,	0x1C
	lsr	temp2
	lsr	temp2
	ldi	ZL,	nBase+5
	st	Z,	temp2
	rcall	checksum
		
	ldi	ZL,	DBase+5	;Month
	ld	temp2,	Z
	lsr	temp2
	lsr	temp2
	lsr	temp2
	lsr	temp2
	lsr	temp2
	ldi	ZL,	DBase+6
	ld	temp,	Z
	andi	temp,	0x03
	lsl	temp
	lsl	temp
	lsl	temp
	add	temp2,	temp
	ldi	ZL,	nBase+3
	st	Z,	temp2
	rcall	checksum
	
	ldi	ZL,	DBase+6	;Year
	ld	temp2,	Z
	andi	temp2,	0xFC
	lsr	temp2
	lsr	temp2
	ldi	ZL,	DBase+7
	ld	temp,	Z
	andi	temp,	0x03
	swap	temp
	lsl	temp
	lsl	temp
	add	temp2,	temp
	ldi	ZL,	nBase+4
	st	Z,	temp2
	rcall	checksum
		
	ldi	ZL,	DBase+7	;Parity Date
	ld	temp,	Z
	andi	temp,	0x04
	lsr	temp
	lsr	temp
	andi	dcfbit,	0x01
	cpse	temp,	dcfbit
	rjmp	parityerror
	
	clr	subsec		;successfully received a DCF telegram
	clr	sek
	;cbi	PORTD,	4
	;sbi	PORTD,	3
	ori	flags,	0x02

	ldi	temp,	8	;copy temporary time to verified time RAM locations
	ldi	ZH,	0
ramcopyloop:
	ldi	ZL,	nBase-1
	add	ZL,	temp
	ld	temp2,	Z
	ldi	ZL,	Base-1
	add	ZL,	temp
	st	Z,	temp2
	dec	temp
	brne	ramcopyloop
	rcall	displaytime
	ret
	
parityerror:
	;cbi	PORTD,	3
	ret
	
;--------------------

checksum:
	ldi	temp,	8
checksumloop:
	sbrc	temp2,	0
	inc	dcfbit
	lsr	temp2
	dec	temp
	brne	checksumloop
	ret

;--------------------

displaytime:
	clr	ZH		;displays date and time
	ldi	temp,	0xC0
	rcall	lcdcmd
	
	ldi	temp,	'2'	;year
	rcall	lcdchr
	ldi	temp,	'0'
	rcall	lcdchr	
	ldi	ZL,	Base+4
	rcall	displayram
	ldi	temp,	'-'	;month
	rcall	lcdchr
	ldi	ZL,	Base+3
	rcall	displayram
	ldi	temp,	'-'	;day
	rcall	lcdchr
	ldi	ZL,	Base+2
	rcall	displayram
	ldi	temp,	' '	;hour
	rcall	lcdchr
	ldi	ZL,	Base+1
	rcall	displayram
	ldi	temp,	':'	;minute
	rcall	lcdchr
	ldi	ZL,	Base+0
	rcall	displayram

	ret

;--------------------

displayram:				;displays packed BCD value from RAM on the LCD
	ld	temp,	Z
	mov	temp2,	temp
	swap	temp
	andi	temp,	0x0F
	andi	temp2,	0x0F
	subi	temp,	-48
	subi	temp2,	-48
	rcall	lcdchr
	mov	temp,	temp2
	rcall	lcdchr
	ret
	
;--------------------

displaypercent:				;zeigt verstrichene Zeit in mm/ss und Prozent an
	ldi	temp,	0x80
	rcall	lcdcmd
	clr	ZH
	ldi	ZL,	Base+1
	ld	temp,	Z
	cpi	temp,	0x18
	brlo	inbounds
	rjmp	schoolsout
inbounds:

	mov	op3a,	sek	;sek
	clr	op3b
	clr	op3c
	clr	op3d

	ldi	ZL,	Base+0	;min
	ld	temp,	Z+
	rcall	bcd2bin
	mov	op1a,	temp2
	clr	op1b
	clr	op1c
	clr	op1d
	ldi	temp,	60
	mov	op2a,	temp
	clr	op2b
	rcall	multiply
	
	ld	temp,	Z+	;hour
	rcall	bcd2bin
	mov	op1a,	temp2
	clr	op1b
	clr	op1c
	clr	op1d
	ldi	temp,	LOW(3600)
	mov	op2a,	temp
	ldi	temp,	HIGH(3600)
	mov	op2b,	temp
	rcall	multiply
	
	ldi	temp,	LOW(Ende)
	cp	op3a,	temp
	ldi	temp,	HIGH(Ende)
	cpc	op3b,	temp
	brlo	beforeend
	rjmp	schoolsout
beforeend:
	
	ldi	temp,	LOW(H6)			;aktuelle Stunde/Pause bestimmen
	cp	op3a,	temp
	ldi	temp,	HIGH(H6)
	cpc	op3b,	temp
	brlo	chkp5
	ldi	temp,	LOW(H6)
	mov	op2a,	temp
	ldi	temp,	HIGH(H6)
	mov	op2b,	temp
	ldi	temp,	LOW(45*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(45*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'H'
	rcall	lcdchr
	ldi	temp,	'6'
	rcall	lcdchr
	rjmp	calcpercent
chkp5:
	ldi	temp,	LOW(P5)
	cp	op3a,	temp
	ldi	temp,	HIGH(P5)
	cpc	op3b,	temp
	brlo	chkh5
	ldi	temp,	LOW(P5)
	mov	op2a,	temp
	ldi	temp,	HIGH(P5)
	mov	op2b,	temp
	ldi	temp,	LOW(5*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(5*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'P'
	rcall	lcdchr
	ldi	temp,	'5'
	rcall	lcdchr
	rjmp	calcpercent
chkh5:
	ldi	temp,	LOW(H5)
	cp	op3a,	temp
	ldi	temp,	HIGH(H5)
	cpc	op3b,	temp
	brlo	chkp4
	ldi	temp,	LOW(H5)
	mov	op2a,	temp
	ldi	temp,	HIGH(H5)
	mov	op2b,	temp
	ldi	temp,	LOW(45*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(45*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'H'
	rcall	lcdchr
	ldi	temp,	'5'
	rcall	lcdchr
	rjmp	calcpercent
chkp4:
	ldi	temp,	LOW(P4)
	cp	op3a,	temp
	ldi	temp,	HIGH(P4)
	cpc	op3b,	temp
	brlo	chkh4
	ldi	temp,	LOW(P4)
	mov	op2a,	temp
	ldi	temp,	HIGH(P4)
	mov	op2b,	temp
	ldi	temp,	LOW(15*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(15*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'P'
	rcall	lcdchr
	ldi	temp,	'4'
	rcall	lcdchr
	rjmp	calcpercent
chkh4:
	ldi	temp,	LOW(H4)
	cp	op3a,	temp
	ldi	temp,	HIGH(H4)
	cpc	op3b,	temp
	brlo	chkp3
	ldi	temp,	LOW(H4)
	mov	op2a,	temp
	ldi	temp,	HIGH(H4)
	mov	op2b,	temp
	ldi	temp,	LOW(45*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(45*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'H'
	rcall	lcdchr
	ldi	temp,	'4'
	rcall	lcdchr
	rjmp	calcpercent
chkp3:
	ldi	temp,	LOW(P3)
	cp	op3a,	temp
	ldi	temp,	HIGH(P3)
	cpc	op3b,	temp
	brlo	chkh3
	ldi	temp,	LOW(P3)
	mov	op2a,	temp
	ldi	temp,	HIGH(P3)
	mov	op2b,	temp
	ldi	temp,	LOW(5*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(5*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'P'
	rcall	lcdchr
	ldi	temp,	'3'
	rcall	lcdchr
	rjmp	calcpercent
chkh3:
	ldi	temp,	LOW(H3)
	cp	op3a,	temp
	ldi	temp,	HIGH(H3)
	cpc	op3b,	temp
	brlo	chkp2
	ldi	temp,	LOW(H3)
	mov	op2a,	temp
	ldi	temp,	HIGH(H3)
	mov	op2b,	temp
	ldi	temp,	LOW(45*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(45*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'H'
	rcall	lcdchr
	ldi	temp,	'3'
	rcall	lcdchr
	rjmp	calcpercent
chkp2:
	ldi	temp,	LOW(P2)
	cp	op3a,	temp
	ldi	temp,	HIGH(P2)
	cpc	op3b,	temp
	brlo	chkh2
	ldi	temp,	LOW(P2)
	mov	op2a,	temp
	ldi	temp,	HIGH(P2)
	mov	op2b,	temp
	ldi	temp,	LOW(15*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(15*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'P'
	rcall	lcdchr
	ldi	temp,	'2'
	rcall	lcdchr
	rjmp	calcpercent
chkh2:
	ldi	temp,	LOW(H2)
	cp	op3a,	temp
	ldi	temp,	HIGH(H2)
	cpc	op3b,	temp
	brlo	chkp1
	ldi	temp,	LOW(H2)
	mov	op2a,	temp
	ldi	temp,	HIGH(H2)
	mov	op2b,	temp
	ldi	temp,	LOW(45*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(45*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'H'
	rcall	lcdchr
	ldi	temp,	'2'
	rcall	lcdchr
	rjmp	calcpercent
chkp1:
	ldi	temp,	LOW(P1)
	cp	op3a,	temp
	ldi	temp,	HIGH(P1)
	cpc	op3b,	temp
	brlo	chkh1
	ldi	temp,	LOW(P1)
	mov	op2a,	temp
	ldi	temp,	HIGH(P1)
	mov	op2b,	temp
	ldi	temp,	LOW(5*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(5*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'P'
	rcall	lcdchr
	ldi	temp,	'1'
	rcall	lcdchr
	rjmp	calcpercent
chkh1:
	ldi	temp,	LOW(H1)
	cp	op3a,	temp
	ldi	temp,	HIGH(H1)
	cpc	op3b,	temp
	brsh	firsthour
	rjmp	schoolsout
firsthour:
	ldi	temp,	LOW(H1)
	mov	op2a,	temp
	ldi	temp,	HIGH(H1)
	mov	op2b,	temp
	ldi	temp,	LOW(45*60)
	mov	op4a,	temp
	ldi	temp,	HIGH(45*60)
	mov	op4b,	temp
	clr	op4c
	clr	op4d
	ldi	temp,	'H'
	rcall	lcdchr
	ldi	temp,	'1'
	rcall	lcdchr
	rjmp	calcpercent

calcpercent:				;Verstrichene Zeit/Prozent ausrechnen

	ldi	temp,	' '
	rcall	lcdchr
	
	sub	op3a,	op2a	;op3 = op3-op2
	sbc	op3b,	op2b
	ldi	temp2,	0
	sbc	op3c,	temp2
	sbc	op3d,	temp2
	
	sbic	PIND,	1	;op3=op4-op3 wenn Restzeitanz.
	rjmp	noinvert
	mov	op2a,	op4a
	mov	op2b,	op4b
	sub	op2a,	op3a
	sbc	op2b,	op3b
	mov	op3a,	op2a
	mov	op3b,	op2b
noinvert:
	
	mov	op2a,	op3a	;verstrichene Zeit in op2 und op4 sichern
	mov	op2b,	op3b
	mov	op5a,	op2a
	mov	op5b,	op2b
	
	ldi	temp2,	BYTE1((1<<20)*1000)
	mov	op3a,	temp2
	ldi	temp2,	BYTE2((1<<20)*1000)
	mov	op3b,	temp2
	ldi	temp2,	BYTE3((1<<20)*1000)
	mov	op3c,	temp2
	ldi	temp2,	BYTE4((1<<20)*1000)
	mov	op3d,	temp2
	rcall	divideround
	
	clr	op3a		;op3 = op1 * op2
	clr	op3b
	clr	op3c
	clr	op3d
	rcall	multiply
	
	mov	op2a,	op5a	;Sicherung nach op2 zurckkopieren
	mov	op2b,	op5b
	
	ldi	temp,	20	;op3 = op3 / 2^20
shiftloop:
	lsr	op3d
	ror	op3c
	ror	op3b
	ror	op3a
	dec	temp
	brne	shiftloop
	brcc	shiftnoround
	ldi	temp2,	1
	add	op3a,	temp2
	ldi	temp2,	0
	adc	op3b,	temp2
	adc	op3c,	temp2
	adc	op3d,	temp2
shiftnoround:

	mov	op5a,	op3a	;op3 (Promille) sichern
	mov	op5b,	op3b
	
	ldi	temp,	60		;Zeit (mm:hh) anzeigen
	mov	op4a,	temp
	clr	op4b
	clr	op4c
	clr	op4d
	mov	op3a,	op2a
	mov	op3b,	op2b
	clr	op3c
	clr	op3d
	rcall	divide
	mov	op2a,	op3a
	mov	op2b,	op3b
	
	mov	op3a,	op1a	;minutes
	mov	op3b,	op1b
	mov	op3c,	op1c
	mov	op3d,	op1d
	ldi	temp,	10
	mov	op4a,	temp
	clr	op4b
	clr	op4c
	clr	op4d
	rcall	divide
	ldi	temp,	0x30
	add	temp,	op1a
	cpi	temp,	0x30
	brne	minnotzero
	ldi	temp,	0x20
minnotzero:
	rcall	lcdchr
	ldi	temp,	0x30
	add	temp,	op3a
	rcall	lcdchr
	ldi	temp,	':'
	rcall	lcdchr
	
	mov	op3a,	op2a	;seconds
	mov	op3b,	op2b
	clr	op3c
	clr	op3d
	ldi	temp,	10
	mov	op4a,	temp
	clr	op4b
	clr	op4c
	clr	op4d
	rcall	divide
	ldi	temp,	0x30
	add	temp,	op1a
	rcall	lcdchr
	ldi	temp,	0x30
	add	temp,	op3a
	rcall	lcdchr
	ldi	temp,	' '
	rcall	lcdchr
	
	ldi	temp,	0x8B		;Prozente anzeigen
	rcall	lcdcmd
	ldi	temp,	'.'
	rcall	lcdchr
	ldi	temp,	0x8D
	rcall	lcdcmd
	ldi	temp,	'%'
	rcall	lcdchr
	
	mov	op3a,	op5a	;Sicherung zurckkopieren
	mov	op3b,	op5b
	clr	op3c
	clr	op3d

	ldi	temp,	0x8C	;0.1
	rcall	lcdcmd
	ldi	temp,	10
	mov	op4a,	temp
	clr	op4b
	clr	op4c
	clr	op4d
	rcall	divide
	ldi	temp,	0x30
	add	temp,	op3a
	rcall	lcdchr
	
	ldi	temp,	0x8A	;1
	rcall	lcdcmd
	ldi	temp,	10
	mov	op3a,	op1a
	mov	op3b,	op1b
	mov	op4a,	temp
	clr	op4b
	clr	op4c
	clr	op4d
	rcall	divide
	ldi	temp,	0x30
	add	temp,	op3a
	rcall	lcdchr

	ldi	temp,	0x88	;10 & 100
	rcall	lcdcmd
	ldi	temp,	10
	mov	op3a,	op1a
	mov	op3b,	op1b
	mov	op4a,	temp
	clr	op4b
	clr	op4c
	clr	op4d
	rcall	divide
	ldi	temp2,	0x30
	add	temp2,	op3a
	ldi	temp,	0x30
	add	temp,	op1a
	cpi	temp,	0x30
	brne	percentnotzero
	ldi	temp,	0x20
	cpi	temp2,	0x30
	brne	percentnotzero
	ldi	temp2,	0x20
percentnotzero:
	rcall	lcdchr
	mov	temp,	temp2
	rcall	lcdchr
	
	ret
	
schoolsout:				;display weekday
	clr	ZH
	ldi	ZL,	Base+5
	ld	temp,	Z
	dec	temp
	mov	temp2,	temp
	lsl	temp	;temp=temp*10
	lsl	temp
	add	temp,	temp2
	lsl	temp
	ldi	ZL,	LOW(0x800-7*10)
	add	ZL,	temp
	ldi	ZH,	HIGH(0x800-7*10)
	
	ldi	temp,	0x80
	rcall	lcdcmd
	ldi	temp,	' '
	rcall	lcdchr
	rcall	lcdchr
	rcall	lcdchr
	ldi	temp2,	10
weekdayloop:
	lpm
	mov	temp,	R0
	rcall	lcdchr
	inc	ZL
	dec	temp2
	brne	weekdayloop
	ldi	temp,	' '
	rcall	lcdchr
	rcall	lcdchr
	rcall	lcdchr
	ret

;--------------------

bcd2bin:				;converts packed BCD (temp) to binary (temp2)
	mov	temp2,	temp
	andi	temp2,	0x0F
bcd2binloop:
	subi	temp,	0x10
	brcs	bcd2binend
	subi	temp2,	-10
	rjmp	bcd2binloop
bcd2binend:
	ret

;--------------------

multiply:				;op3 = op3 + (op1 * op2)
	ldi	temp,	16

multiloop:
	lsr	op2b
	ror	op2a
	brcc	multinoadd
	add	op3a,	op1a
	adc	op3b,	op1b
	adc	op3c,	op1c
	adc	op3d,	op1d
multinoadd:
	lsl	op1a
	rol	op1b
	rol	op1c
	rol	op1d
	dec	temp
	brne	multiloop
	ret

;--------------------

divideround:				;op1 = op3 / op4 (rounded!)
	clr	op1a
	clr	op1b
	clr	op1c
	clr	op1d
	ldi	temp,	1
divroundpreloop:
	lsl	op4a
	rol	op4b
	rol	op4c
	rol	op4d
	inc	temp
	cp	op4a,	op3a
	cpc	op4b,	op3b
	cpc	op4c,	op3c
	cpc	op4d,	op3d
	brlo	divroundpreloop
	
divroundloop:
	lsl	op1a
	rol	op1b
	rol	op1c
	rol	op1d
	cp	op3a,	op4a
	cpc	op3b,	op4b
	cpc	op3c,	op4c
	cpc	op3d,	op4d
	brlo	divroundnoinc
	
	ldi	temp2,	1
	add	op1a,	temp2
	ldi	temp2,	0
	adc	op1b,	temp2
	adc	op1c,	temp2
	adc	op1d,	temp2
	sub	op3a,	op4a
	sbc	op3b,	op4b
	sbc	op3c,	op4c
	sbc	op3d,	op4d
	
divroundnoinc:
	lsr	op4d
	ror	op4c
	ror	op4b
	ror	op4a
	dec	temp
	brne	divroundloop
	
	brcc	divround	;runden
	ldi	temp2,	1
	add	op4a,	temp2
	ldi	temp2,	0
	adc	op4b,	temp2
	adc	op4c,	temp2
	adc	op4d,	temp2
divround:
	cp	op3a,	op4a
	cpc	op3b,	op4b
	cpc	op3c,	op4c
	cpc	op3d,	op4d
	brlo	divroundend
	ldi	temp2,	1
	add	op1a,	temp2
	ldi	temp2,	0
	adc	op1b,	temp2
	adc	op1c,	temp2
	adc	op1d,	temp2
divroundend:
	ret

;--------------------

divide:					;op1 = op3 / op4 ; op3 = op3 % op4
	clr	op1a
	clr	op1b
	clr	op1c
	clr	op1d
	ldi	temp,	1
divpreloop:
	lsl	op4a
	rol	op4b
	rol	op4c
	rol	op4d
	inc	temp
	cp	op4a,	op3a
	cpc	op4b,	op3b
	cpc	op4c,	op3c
	cpc	op4d,	op3d
	brlo	divpreloop
	
divloop:
	lsl	op1a
	rol	op1b
	rol	op1c
	rol	op1d
	cp	op3a,	op4a
	cpc	op3b,	op4b
	cpc	op3c,	op4c
	cpc	op3d,	op4d
	brlo	divnoinc
	
	ldi	temp2,	1
	add	op1a,	temp2
	ldi	temp2,	0
	adc	op1b,	temp2
	adc	op1c,	temp2
	adc	op1d,	temp2
	sub	op3a,	op4a
	sbc	op3b,	op4b
	sbc	op3c,	op4c
	sbc	op3d,	op4d
	
divnoinc:
	lsr	op4d
	ror	op4c
	ror	op4b
	ror	op4a
	dec	temp
	brne	divloop
	
	ret

